// ======================================================================
//
// MenuResource.h
// jeff grills
//
// copyright 1997 origin systems inc
//
// ======================================================================

#include "First.h"
#include "MenuResource.h"

#include "Bitmap.h"
#include "Font.h"
#include "Iff.h"
#include "StringFile.h"
#include "MenuElement.h"
#include "Mouse.h"
//#include "Sound.h"

// ======================================================================

const Tag TAG_NLOP                   = TAG(N,L,O,P);
const Tag TAG_SIAE                   = TAG(S,I,A,E); // Shared Integer Animation Entry
const Tag TAG_GEOV                   = TAG(G,E,O,V); // generated events on 
const Tag TAG_SMPL                   = TAG(S,M,P,L); // sound
const Tag TAG_PATH                   = TAG(P,A,T,H); //
const Tag TAG_PRTY                   = TAG(P,R,T,Y); //
const Tag TAG_VOLU                   = TAG(V,O,L,U); //
const Tag TAG_PAN                    = TAG3(P,A,N);  //
const Tag TAG_SEVN                   = TAG(S,E,V,N); //
const Tag TAG_STMS                   = TAG(S,T,M,S); // SetMouse
const Tag TAG_EMOS                   = TAG(E,M,O,S); // 
const Tag TAG_LANG                   = TAG(L,A,N,G);
const int MENU_INT_ANIM_STOP         = -1;
const int MENU_INT_ANIM_KEEP_LOOPING = -2;

// ======================================================================

MenuResource::MenuResource(Type newType, MenuElement *newOwner)
{
	type = newType;
	owner = newOwner;
	id = NULL;
	next = NULL;
}

// ----------------------------------------------------------------------

MenuResource::~MenuResource(void)
{
	delete [] id;
	id = NULL;
	
	owner = NULL;

	next = NULL;
}

// ----------------------------------------------------------------------

void MenuResource::load(Iff *iff)
{
	if (iff->seekChunk(TAG_NAME, Iff::FlagOptional))
	{
		id = MEM_OWN(iff->read_char(Iff::ReadRest));
		iff->exitChunk();
	}
}

// ----------------------------------------------------------------------

MenuResource *MenuResource::findSharedResource(MenuResource::Type findType, const char *findId)
{
	if (type == findType && id && strcmp(id, findId) == 0)
		return this;

	return NULL;
}

// ----------------------------------------------------------------------

void MenuResource::menuEvent(int, MenuElement *)
{
}

// ----------------------------------------------------------------------

void MenuResource::update(void)
{
}

// ======================================================================

MenuResourceBitmap::MenuResourceBitmap(MenuElement *newOwner, Iff *iff)
: MenuResource(TypeBitmap, newOwner)
{
	bitmap = NULL;
	load(iff);
}

// ----------------------------------------------------------------------

MenuResourceBitmap::~MenuResourceBitmap(void)
{
	delete bitmap;
	bitmap = NULL;
}

// ----------------------------------------------------------------------

void MenuResourceBitmap::load(Iff *iff)
{
	if (iff)
	{
		MenuResource::load(iff);
		bitmap = NEW Bitmap(*iff);
	}
}

// ======================================================================

MenuResourceFont::MenuResourceFont(MenuElement *newOwner, Iff *iff)
: MenuResource(TypeFont, newOwner)
{
	font = NULL;
	load(iff);
}

// ----------------------------------------------------------------------

MenuResourceFont::~MenuResourceFont(void)
{
	delete font;
	font = NULL;
}

// ----------------------------------------------------------------------

void MenuResourceFont::load(Iff *iff)
{
	int r, g, b;
	char *name;

	if (iff)
	{
		MenuResource::load(iff);

		iff->seekChunk(TAG_FONT);

			r = iff->read_int32();
			g = iff->read_int32();
			b = iff->read_int32();
			name = iff->read_char(Iff::ReadRest);

			font = NEW Font(name,r,g,b);
			delete [] name;

		iff->exitChunk();
	}
}

// ======================================================================

MenuResourceLanguage::MenuResourceLanguage(MenuElement *newOwner, Iff *iff)
: MenuResource(TypeLanguage, newOwner)
{
	partition = NULL;
	load(iff);
}

// ----------------------------------------------------------------------

MenuResourceLanguage::~MenuResourceLanguage(void)
{
	delete partition;
	partition = NULL;
}

// ----------------------------------------------------------------------

void MenuResourceLanguage::load(Iff *iff)
{
	char *name;

	if (iff)
	{
		MenuResource::load(iff);

		iff->seekChunk(TAG_LANG);

			name = MEM_OWN(iff->read_char(Iff::ReadRest));

			partition = NEW StringFile(name);
			delete [] name;

		iff->exitChunk();
	}
}

// ======================================================================

MenuResourceInteger::MenuResourceInteger(MenuElement *newOwner, Iff *iff)
: MenuResource(TypeInteger, newOwner)
{
	init();
	load(iff);
}

// ----------------------------------------------------------------------

MenuResourceInteger::~MenuResourceInteger(void)
{
	delete [] changedEvent;
	changedEvent   = NULL;
	delete [] changeEvent;
	changeEvent    = NULL;
	delete [] eventsOnValues;
	eventsOnValues = NULL;
	init();
}

// ----------------------------------------------------------------------

void MenuResourceInteger::init(void)
{
	looping           = true;
	changedEvent      = NULL;
	numChangedEvents  = 0;
	minState          = 0;
	maxState          = 1;
	currentState      = 0;
	changeEvents      = 0;
	changeEvent       = NULL;
	numEventsOnValues = 0;
	eventsOnValues    = NULL;
}
// ----------------------------------------------------------------------

void MenuResourceInteger::load(Iff *iff)
{
	int i;

	if (iff)
	{
		MenuResource::load(iff);

		numChangedEvents = iff->blockCount(Iff::FlagChunk, TAG_CHNG);
		if (numChangedEvents > 0)
		{
			changedEvent = NEW int[numChangedEvents];
			memset(changedEvent, '\0', sizeof(*changedEvent) * numChangedEvents);
			for (i = 0; iff->seekChunk(TAG_CHNG, Iff::FlagOptional); i++)
			{
				changedEvent[i] = iff->read_int32();
				iff->exitChunk();
			}
		}

		if (iff->seekChunk(TAG_NLOP, Iff::FlagOptional))
		{
			looping = false;
			iff->exitChunk();
		}

		if (iff->seekChunk(TAG_MINS, Iff::FlagOptional))
		{
			currentState = minState = iff->read_int32();
			iff->exitChunk();
		}

		if (iff->seekChunk(TAG_MAXS, Iff::FlagOptional))
		{
			maxState = iff->read_int32();
			iff->exitChunk();
		}

		if (iff->seekChunk(TAG_CURS, Iff::FlagOptional))
		{
			currentState = iff->read_int32();
			iff->exitChunk();
		}

		changeEvents = iff->blockCount(Iff::FlagChunk | Iff::FlagNamed, TAG_CNGS);
		if (changeEvents)
		{
			changeEvent = NEW ChangeEvent[changeEvents];

			for (i = 0; iff->seekChunk(TAG_CNGS, Iff::FlagOptional); i++)
			{
				changeEvent[i].event    = iff->read_int32();
				changeEvent[i].amount   = iff->read_int32();
				changeEvent[i].absolute = (iff->read_int32() != 0);
				iff->exitChunk(); // 'CNGS'
			}
		}

		// these 'events' are generated when the "shared" integer is equal to one of these
		numEventsOnValues = iff->blockCount(Iff::FlagChunk, TAG_GEOV);
		if (numEventsOnValues)
		{
			eventsOnValues = NEW EventOnValue[numEventsOnValues];

			for (i = 0; iff->seekChunk(TAG_GEOV, Iff::FlagOptional); i++)
			{
				eventsOnValues[i].value = iff->read_int32();
				eventsOnValues[i].event = iff->read_int32();
				iff->exitChunk(); // 'GEON'
			}
		}
	}
}

// ----------------------------------------------------------------------

int MenuResourceInteger::getValue(void) const
{
	return currentState;
}

// ----------------------------------------------------------------------

void MenuResourceInteger::setValue(int newValue)
{
	int i;

	if (newValue < minState)
		if (looping)
			newValue = maxState;
		else
			newValue = minState;

	if (newValue > maxState)
		if (looping)
			newValue = minState;
		else
			newValue = maxState;

	if (newValue != currentState)
	{
		DPRINT(("MenuResInt '%s' new value %d\n", id, newValue));
		currentState = newValue;

		if (changedEvent)
			for (i = 0; i < numChangedEvents; i++)
				owner->menuEvent(changedEvent[i], owner);

		// see if this new value is supposed to "trigger" any other events
		for (i = 0; i < numEventsOnValues; i++)
			if (eventsOnValues[i].value == currentState)
				owner->menuEvent(eventsOnValues[i].event, owner);
	}
}

// ----------------------------------------------------------------------

void MenuResourceInteger::menuEvent(int id, MenuElement *)
{
	int i;

	for (i = 0; i < changeEvents && changeEvent[i].event != id; i++)
		;

	if (i < changeEvents)
	{
		if (changeEvent[i].absolute)
			setValue(changeEvent[i].amount);
		else
			setValue(getValue() + changeEvent[i].amount);
	}
}

// ======================================================================

MenuResourceIntegerAnimation::MenuResourceIntegerAnimation(MenuElement *newOwner, Iff *iff)
: MenuResourceInteger(newOwner, NULL)
{
	timer;
	currentAnimation    = 0;
	currentFrame        = 0;
	animation           = NULL;
	numAnims            = 0;
	type                = TypeIntegerAnimation;
	load(iff);
}

// ----------------------------------------------------------------------

MenuResourceIntegerAnimation::~MenuResourceIntegerAnimation(void)
{
	int i;

	for (i = 0; i < numAnims; i++)
	{
		delete []animation[i];
		animation[i] = NULL;
	}
	delete []animation;
	animation = NULL;
}

// ----------------------------------------------------------------------

void MenuResourceIntegerAnimation::load(Iff *iff)
{
	real seconds;
	real hz;
	int  i;
	int  n;

	if (iff)
	{
		MenuResourceInteger::load(iff);

		if (iff->seekChunk(TAG_SEC, Iff::FlagOptional))
		{
			seconds = iff->read_float();
			timer.setTime(seconds);
			iff->exitChunk();
		}
		else
		{
			(void)iff->seekChunk(TAG_HZ);
			hz = iff->read_float();
			timer.setHz(hz);
			iff->exitChunk();
		}

		// load in the entries
		if ((numAnims = iff->blockCount(Iff::FlagChunk, TAG_SIAE)) <= 0)
			Fatal("MenuResourceIntegerAnimation::load must contain at least 1 entry.");
		animation = NEW int *[numAnims];
		for (i = 0; iff->seekChunk(TAG_SIAE, Iff::FlagOptional); i++)
		{
			n = iff->chunkLength(sizeof(int));
			animation[i] = NEW int[n];
			iff->read_int32(n, animation[i]);
			iff->exitChunk(); // 'SIAE'
		}

		// start with the correct animation
		currentAnimation = getValue();
	}
}

// ----------------------------------------------------------------------

void MenuResourceIntegerAnimation::update(void)
{
	real t;
	int  f;

	// see if our shared integer changed
	f = getValue();
	if (f != currentAnimation)
	{
		currentAnimation = f;
		currentFrame     = 0;
	}

	// update our current animation
	t  = Clock::frameTime();
	while (timer.updateSub(t))
	{
		t = CONST_REAL(0);
		currentFrame++;
		switch (animation[currentAnimation][currentFrame])
		{
			case MENU_INT_ANIM_STOP:         // stop
				currentFrame--;
				break;
			case MENU_INT_ANIM_KEEP_LOOPING: // keep looping
				currentFrame = 0;
				break;
		}
	}

	MenuResourceInteger::update();
}

// ----------------------------------------------------------------------
// reset the animation
void MenuResourceIntegerAnimation::setValue(int newValue)
{
	MenuResourceInteger::setValue(newValue);
	currentFrame = 0;
}

// ----------------------------------------------------------------------

int MenuResourceIntegerAnimation::getFrame(void) const
{
	return animation[currentAnimation][currentFrame];
}


// ======================================================================

MenuSound::MenuSound(MenuElement *newOwner, Iff *iff)
: MenuResource(TypeSound, newOwner)
{
	init();
	load(iff);
}

// ----------------------------------------------------------------------

MenuSound::~MenuSound(void)
{
}

// ----------------------------------------------------------------------

void MenuSound::init(void)
{
	type         = TypeSound;
	event        = 0;
	sampleNumber = -1;
#if 0
	path         = FNP_digital;
#endif
	setPriority  = 0;
	volume       = CONST_REAL(100.0f);
	pan          = CONST_REAL(0.0f);
}

// ----------------------------------------------------------------------

void MenuSound::load(Iff *iff)
{
	if (iff)
	{
		MenuResource::load(iff);

		iff->seekChunk(TAG_SEVN);
		event = iff->read_int32();
		iff->exitChunk(); // TAG_SEVN

		iff->seekChunk(TAG_SMPL);
		sampleNumber = iff->read_int32();
		iff->exitChunk(); // TAG_SMPL

		if (iff->seekChunk(TAG_PATH, Iff::FlagOptional))
		{
#if 0
			path = static_cast<FileNamePrefix>(iff->read_int32());
#endif
			iff->exitChunk(); // TAG_PATH
		}

		if (iff->seekChunk(TAG_PRTY, Iff::FlagOptional))
		{
			setPriority = iff->read_int32();
			iff->exitChunk(); // TAG_PRTY
		}

		if (iff->seekChunk(TAG_VOLU, Iff::FlagOptional))
		{
			volume = iff->read_float();
			iff->exitChunk(); // TAG_VOLU
		}

		if (iff->seekChunk(TAG_PAN, Iff::FlagOptional))
		{
			pan = iff->read_float();
			iff->exitChunk(); // TAG_PAN
		}
	}
}

// ----------------------------------------------------------------------

void MenuSound::menuEvent(int id, MenuElement *)
{
	if (id == event)
	{
#if 0
		Sound::play(sampleNumber, path, static_cast<Sound::Priority>(setPriority), volume, pan);
#endif
	}
}

// ======================================================================

MenuSetMouse::MenuSetMouse(MenuElement *newOwner, Iff *iff)
: MenuResource(TypeSetMouse, newOwner)
{
	init();
	load(iff);
}

// ----------------------------------------------------------------------

MenuSetMouse::~MenuSetMouse(void)
{
	delete []mousePos;
	mousePos = NULL;
	init();
}

// ----------------------------------------------------------------------

void MenuSetMouse::init(void)
{
	mousePos       = NULL;
	numMouseEvents = 0;
}

// ----------------------------------------------------------------------

void MenuSetMouse::load(Iff *iff)
{
	int i, x, y;

	if (iff)
	{
		MenuResource::load(iff);

		// do we need to set the mouse immediately?
		if (iff->seekChunk(TAG_STMS, Iff::FlagOptional))
		{
			x = iff->read_int32();
			y = iff->read_int32();
			Mouse::gotoXY(x, y);
			iff->exitChunk(); // 'STMS'
		}

		numMouseEvents = iff->blockCount(Iff::FlagChunk, TAG_EMOS);
		if (numMouseEvents > 0)
		{
			mousePos = NEW MousePosition[numMouseEvents];
			for (i = 0; i < numMouseEvents; i++)
			{
				iff->seekChunk(TAG_EMOS);
				mousePos[i].eventId = iff->read_int32();
				mousePos[i].x       = iff->read_int32();
				mousePos[i].y       = iff->read_int32();
				iff->exitChunk(); // 'EMOS'
			}
		}
	}
}

// ----------------------------------------------------------------------
// check to see if we need to set the mouse position

void MenuSetMouse::menuEvent(int id, MenuElement *)
{
	int i;

	for (i = 0; i < numMouseEvents; i++)
		if (mousePos[i].eventId == id)
			Mouse::gotoXY(mousePos[i].x, mousePos[i].y);
}

// ======================================================================
